# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-08-01 18:35
from __future__ import unicode_literals

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Conta',
            fields=[
                ('id', models.AutoField(
                    auto_created=True,
                    primary_key=True,
                    serialize=False,
                    verbose_name='ID')),
                ('name', models.CharField(
                    max_length=255,
                    verbose_name='Título da Conta:')),
                ('description', models.TextField(
                    blank=True, default='', verbose_name='Descrição:')),
                ('owner', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='contas', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Lancamento',
            fields=[
                ('id', models.AutoField(
                    auto_created=True,
                    primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('name', models.CharField(
                    max_length=255,
                    verbose_name='Título do Lançamento:')),
                ('date', models.DateField(
                    default=datetime.date.today,
                    verbose_name='Data do Lançamento:')),
                ('lanc_tp', models.IntegerField(
                    choices=[(1, 'Débito'), (2, 'Crédito')],
                    default=1, verbose_name='Tipo de Lançamento:')),
                ('value', models.DecimalField(
                    decimal_places=2, max_digits=12,
                    verbose_name='Valor do Lançamento:')),
                ('conta', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='lancamentos',
                    to='contas_orama.Conta')),
                ('owner', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='lancamentos',
                    to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
