"""
Criado automaticamente pelo comando:
$ django-admin startapp contasOrama
"""
from django.apps import AppConfig


class ContasoramaConfig(AppConfig):
    """
    Criado automaticamente pelo comando:
    $ django-admin startapp contasOrama
    """
    name = 'contas_orama'
